@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2010-2015  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2005, 2007, 2010-2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -55,8 +55,13 @@ done
 
 echo "I: checking that named-checkconf -z catches missing hint file"
 ret=0
-$CHECKCONF -z hint-nofile.conf > hint-nofile.out 2>&1 && ret=1
-grep "could not configure root hints from 'nonexistent.db': file not found" hint-nofile.out > /dev/null || ret=1
+$CHECKCONF -z hint-nofile.conf > /dev/null 2>&1 && ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+echo "I: checking that named-checkconf catches range errors"
+ret=0
+$CHECKCONF range.conf > /dev/null 2>&1 && ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -138,6 +143,12 @@ n=`$CHECKCONF inline-bad.conf 2>&1 | grep "missing 'file' entry" | wc -l`
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
+echo "I: checking named-checkconf DLZ warnings"
+ret=0
+$CHECKCONF dlz-bad.conf 2>&1 | grep "'dlz' and 'database'" > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
 echo "I: checking for missing key directory warning"
 ret=0
 rm -rf test.keydir
@@ -152,6 +163,20 @@ n=`$CHECKCONF warn-keydir.conf 2>&1 | grep "key-directory" | wc -l`
 [ $n -eq 0 ] || ret=1
 rm -rf test.keydir
 if [ $ret != 0 ]; then echo "I:failed"; fi
+
+echo "I: checking that named-checkconf -z catches conflicting ttl with max-ttl"
+ret=0
+$CHECKCONF -z max-ttl.conf > check.out 2>&1
+grep 'TTL 900 exceeds configured max-zone-ttl 600' check.out > /dev/null 2>&1 || ret=1
+grep 'TTL 900 exceeds configured max-zone-ttl 600' check.out > /dev/null 2>&1 || ret=1
+grep 'TTL 900 exceeds configured max-zone-ttl 600' check.out > /dev/null 2>&1 || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
+status=`expr $status + $ret`
+
+echo "I: checking that named-checkconf -z catches invalid max-ttl"
+ret=0
+$CHECKCONF -z max-ttl-bad.conf > /dev/null 2>&1 && ret=1
+if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
 status=`expr $status + $ret`
 
 echo "I: checking that named-checkconf -z skips zone check with alternate databases"
@@ -160,51 +185,9 @@ $CHECKCONF -z altdb.conf > /dev/null 2>&1 || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
 status=`expr $status + $ret`
 
-echo "I: check that check-names fails as configured"
+echo "I: checking that named-checkconf -z skips zone check with DLZ"
 ret=0
-$CHECKCONF -z check-names-fail.conf > checkconf.out1 2>&1 && ret=1
-grep "near '_underscore': bad name (check-names)" checkconf.out1 > /dev/null || ret=1
-grep "zone check-names/IN: loaded serial" < checkconf.out1 > /dev/null && ret=1
-if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
-status=`expr $status + $ret`
-
-echo "I: check that check-mx fails as configured"
-ret=0
-$CHECKCONF -z check-mx-fail.conf > checkconf.out2 2>&1 && ret=1
-grep "near '10.0.0.1': MX is an address" checkconf.out2 > /dev/null || ret=1
-grep "zone check-mx/IN: loaded serial" < checkconf.out2 > /dev/null && ret=1
-if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
-status=`expr $status + $ret`
-
-echo "I: check that check-dup-records fails as configured"
-ret=0
-$CHECKCONF -z check-dup-records-fail.conf > checkconf.out3 2>&1 && ret=1
-grep "has semantically identical records" checkconf.out3 > /dev/null || ret=1
-grep "zone check-dup-records/IN: loaded serial" < checkconf.out3 > /dev/null && ret=1
-if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
-status=`expr $status + $ret`
-
-echo "I: check that check-mx fails as configured"
-ret=0
-$CHECKCONF -z check-mx-fail.conf > checkconf.out4 2>&1 && ret=1
-grep "failed: MX is an address" checkconf.out4 > /dev/null || ret=1
-grep "zone check-mx/IN: loaded serial" < checkconf.out4 > /dev/null && ret=1
-if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
-status=`expr $status + $ret`
-
-echo "I: check that check-mx-cname fails as configured"
-ret=0
-$CHECKCONF -z check-mx-cname-fail.conf > checkconf.out5 2>&1 && ret=1
-grep "MX.* is a CNAME (illegal)" checkconf.out5 > /dev/null || ret=1
-grep "zone check-mx-cname/IN: loaded serial" < checkconf.out5 > /dev/null && ret=1
-if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
-status=`expr $status + $ret`
-
-echo "I: check that check-srv-cname fails as configured"
-ret=0
-$CHECKCONF -z check-srv-cname-fail.conf > checkconf.out6 2>&1 && ret=1
-grep "SRV.* is a CNAME (illegal)" checkconf.out6 > /dev/null || ret=1
-grep "zone check-mx-cname/IN: loaded serial" < checkconf.out6 > /dev/null && ret=1
+$CHECKCONF -z altdlz.conf > /dev/null 2>&1 || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
 status=`expr $status + $ret`
 
