@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Copyright (C) 2011-2013  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2011-2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -26,20 +26,22 @@ QPERF=`sh qperf.sh`
 
 sh clean.sh
 
-# set up test policy zones.  bl-2 is used to check competing zones.
-#	bl-{given,disabled,passthru,no-data,nxdomain,cname,wildcard,garden}
-#	are used to check policy overrides in named.conf.
-#	NO-OP is an obsolete synonym for PASSHTRU
-for NM in '' -2 -given -disabled -passthru -no-op -nodata -nxdomain -cname -wildcname -garden; do
+# set up test policy zones.
+#   bl is the main test zone
+#   bl-2 is used to check competing zones.
+#   bl-{given,disabled,passthru,no-data,nxdomain,cname,wildcard,garden,
+#	    drop,tcp-only} are used to check policy overrides in named.conf.
+#   NO-OP is an obsolete synonym for PASSHTRU
+for NM in '' -2 -given -disabled -passthru -no-op -nodata -nxdomain -cname -wildcname -garden -drop -tcp-only; do
     sed -e "/SOA/s/blx/bl$NM/g" ns3/base.db >ns3/bl$NM.db
 done
 
 # sign the root and a zone in ns2
-../../../tools/genrandom 400 random.data
+test -r $RANDFILE || $GENRANDOM 400 $RANDFILE
 
 # $1=directory, $2=domain name, $3=input zone file, $4=output file
 signzone () {
-    KEYNAME=`$KEYGEN -q -r random.data -b 512 -K $1 $2`
+    KEYNAME=`$KEYGEN -q -r $RANDFILE -b 512 -K $1 $2`
     cat $1/$3 $1/$KEYNAME.key > $1/tmp
     $SIGNER -Pp -K $1 -o $2 -f $1/$4 $1/tmp >/dev/null
     sed -n -e 's/\(.*\) IN DNSKEY \([0-9]\{1,\} [0-9]\{1,\} [0-9]\{1,\}\) \(.*\)/trusted-keys {"\1" \2 "\3";};/p' $1/$KEYNAME.key >>trusted.conf
@@ -48,18 +50,22 @@ signzone () {
 signzone ns2 tld2s. base-tld2s.db tld2s.db
 
 
-# Performance checks.
+# Performance and a few other checks.
 cat <<EOF >ns5/rpz-switch
 response-policy {
-	zone "bl0"; zone "bl1"; zone "bl2";
+	zone "bl0"; zone "bl1"; zone "bl2"; zone "bl3"; zone "bl4";
+	zone "bl5"; zone "bl6"; zone "bl7"; zone "bl8"; zone "bl9";
+	zone "bl10"; zone "bl11"; zone "bl12"; zone "bl13"; zone "bl14";
+	zone "bl15"; zone "bl16"; zone "bl17"; zone "bl18"; zone "bl19";
     } recursive-only no
-	max-policy-ttl 90
-	# min-ns-dots 0
-	break-dnssec yes;
+    max-policy-ttl 90
+    break-dnssec yes
+    qname-wait-recurse no
+    ;
 EOF
 
 cat <<EOF >ns5/example.db
-\$TTL	120
+\$TTL	300
 @	SOA	.  hostmaster.ns.example.tld5. ( 1 3600 1200 604800 60 )
 	NS	ns
 	NS	ns1
@@ -68,15 +74,16 @@ ns1	A	10.53.0.5
 EOF
 
 cat <<EOF >ns5/bl.db
-\$TTL	120
+\$TTL	300
 @		SOA	.  hostmaster.ns.blperf. ( 1 3600 1200 604800 60 )
-		NS	ns
-ns		A	10.53.0.5
+		NS	ns.tld5.
 
-; used only in failure for "recursive-only no" in #8 test5
-a3-5.tld2	CNAME	*.
+; for "qname-wait-recurse no" in #35 test1
+x.servfail	A	35.35.35.35
+; for "recursive-only no" in #8 test5
+a3-5.tld2	CNAME	.
 ; for "break-dnssec" in #9 & #10 test5
-a3-5.tld2s	CNAME	*.
+a3-5.tld2s	CNAME	.
 ; for "max-policy-ttl 90" in #17 test5
 a3-17.tld2	500 A	17.17.17.17
 
@@ -85,8 +92,7 @@ ns1.x.rpz-nsdname	CNAME	.
 EOF
 
 if test -n "$QPERF"; then
-    # do not build the full zones if we will not use them to avoid the long
-    # time otherwise required to shut down the server
+    # Do not build the full zones if we will not use them.
     $PERL -e 'for ($val = 1; $val <= 65535; ++$val) {
 	printf("host-%05d\tA    192.168.%d.%d\n", $val, $val/256, $val%256);
 	}' >>ns5/example.db
@@ -110,3 +116,5 @@ $PERL -e 'for ($cnt = $val = 1; $cnt <= 3000; ++$cnt) {
 	printf("host-%05d.example.tld5 A\n", $val);
 	$val = ($val * 9 + 32771) % 65536;
 	}' >ns5/requests
+
+cp ns2/bl.tld2.db.in ns2/bl.tld2.db
